class GuiActions:
    tab = "↔TAB"
    leave   = "Sair"
    edit = "Editar"
    move = "Mover"
    principal = "Principal"
    github = "Web   "
    help = "Ajuda"
    download = "Baixar"
    activate = "Ativar"
    navigate = "←↓→"
    uncheck = "Desmarcar"
    search = "Busca"
    config = "Config"
    grade = "Pontuar"
    # tester frame
    time_limit = "Tempo"
    evaluate_tester = "Avaliar "
    execute_tester  = "Rodar"
    calibrate = "Calibrar"
    all = "Todos"

class GuiKeys:
    left = "A"
    right = "D"
    down = "S"
    up = "W"

    left2 = "H"
    right2 = "L"
    down2 = "J"
    up2 = "K"
    calibrate = "C"

    palette = "c"
    images = "I"
    down_task = "b"
    activate = "\n"
    key_help = "?"
    
    # rate = "R"
    set_lang_drafts = "R"
    open_url = "u"
    key_quit = "q"
    edit= "e"
    colors = "C"
    borders = "B"
    search = "/"
    graph = "v"
    grade_play = "p"
    execute_tester = "r" # Interact in manual input
    diff = "d"
    evaluate = "\n"
    toggle_main = "\t"
    quit   = "q"
    lock = "u"
    limite  = "t"
    expand = ">"
    collapse = "<"
    mass = "Y"
    hidden = "E"